/* xdwapi sample

Function :
  Creates DocuWorks document from an image file.
How to use :
  xdwsmpl image [document]

  image : BMP, JPEG or TIFF format file (in).
  document : DocuWorks document (out).
  if document is omitted, DocuWorks Desk is started and the document is created there.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <windows.h>

#include <xdw_api.h>

void print_error(int code)
{
	switch (code) {
	case XDW_E_NOT_INSTALLED:
		fprintf(stderr, "DocuWorks not installed.");
		break;
	case XDW_E_FILE_NOT_FOUND:
		fprintf(stderr, "File not found.");
		break;
	case XDW_E_FILE_EXISTS:
		fprintf(stderr, "File exists.");
		break;
	case XDW_E_ACCESSDENIED:
	case XDW_E_INVALID_ACCESS:
	case XDW_E_INVALID_NAME:
	case XDW_E_BAD_NETPATH:
		fprintf(stderr, "File not opened.");
		break;
	case XDW_E_BAD_FORMAT:
		fprintf(stderr, "File bad format.");
		break;
	default:
		fprintf(stderr, "Error.");
		break;
	}
}

int main(int argc, char** argv)
{
	int api_result = 0;

	if (argc != 3 && argc != 2) {
		fprintf(stderr, "How to use: xdwsmpl image [document]");
		return 0;
	}

	char in_path[_MAX_PATH];
	_fullpath(in_path, argv[1], _MAX_PATH);

	char out_path[_MAX_PATH];
	if (argc == 3) {
		_fullpath(out_path, argv[2], _MAX_PATH);
	} else {
		// Use XDW_GetInformation to get the directory for inputting DocuWorks documents.
		// Create DocuWorks document in the directory.
		// After DocuWorks Desk starts, the document presents.
		api_result = XDW_GetInformation(
			XDW_GI_DWINPUTPATH, out_path, _MAX_PATH, NULL);
		if (api_result < 0) {
			print_error(api_result);
			return 0;
		}
		char fname[_MAX_FNAME];
		_splitpath(in_path, NULL, NULL, fname, NULL);
		strcat(out_path, "\\");
		strcat(out_path, fname);
		strcat(out_path, ".xdw");
	}

	// Use XDW_CreateXdwFromImageFile to create DocuWorks document from the image file.
	// Set nFitImage up as 1 to set the page size to be the same as the image size.
	XDW_CREATE_OPTION option = {
		sizeof(XDW_CREATE_OPTION), 1, 0, 0, 0, 0, 0, 0 };
	api_result = XDW_CreateXdwFromImageFile(
		in_path, out_path, &option);
	if (api_result < 0) {
		print_error(api_result);
		return 0;
	}

	if (argc == 2) {
		// Use XDW_GetInformation to get DocuWorks Desk program path.
		char desk_path[_MAX_PATH];
		api_result = XDW_GetInformation(
			XDW_GI_DWDESKPATH, desk_path, _MAX_PATH, NULL);
		if (api_result < 0) {
			print_error(api_result);
			return 0;
		}
		WinExec(desk_path, SW_SHOW);
	}

	return 1;
}
