/* xdwapi sample 2

Function :
  Gets the text data of a page at the standard output.
How to use :
  xdwsmpl2 document [page]

  document : DocuWorks document (in).
  page : Specified page (in). If omitted, all of pages are covered.
*/

#include <stdio.h>
#include <stdlib.h>
#include <io.h>

#include <windows.h>

#include <xdw_api.h>

void print_error(int code)
{
	switch (code) {
	case XDW_E_NOT_INSTALLED:
		fprintf(stderr, "DocuWorks not installed.");
		break;
	case XDW_E_FILE_NOT_FOUND:
		fprintf(stderr, "File not found.");
		break;
	case XDW_E_FILE_EXISTS:
		fprintf(stderr, "File exists.");
		break;
	case XDW_E_ACCESSDENIED:
	case XDW_E_INVALID_NAME:
	case XDW_E_BAD_NETPATH:
		fprintf(stderr, "File not opened.");
		break;
	case XDW_E_BAD_FORMAT:
		fprintf(stderr, "File bad format.");
		break;
	case XDW_E_INVALID_ACCESS:
		fprintf(stderr, "Invalid access.");
		break;
	default:
		fprintf(stderr, "Error.");
		break;
	}
}

int print_text(XDW_DOCUMENT_HANDLE h, int page)
{
	// Use XDW_GetPageInformation to check the page type.
	XDW_PAGE_INFO info = { sizeof(XDW_PAGE_INFO), 0, 0, 0, 0, 0, 0, 0 };
	int result = XDW_GetPageInformation(h, page, &info);
	if (result < 0) return result;

	if (info.nPageType == XDW_PGT_FROMAPPL
		|| info.nPageType == XDW_PGT_FROMIMAGE) {
		// Use XDW_GetPageText to get text data.
		char temp_path[_MAX_PATH];
		strcpy(temp_path, "xdw");
		_mktemp(temp_path);
		int result = XDW_GetPageText(h, page, temp_path, NULL);
		if (result < 0) return result;
		FILE *temp_file = fopen(temp_path, "r");
		int c = fgetc(temp_file);
		while (c != EOF) {
			putchar(c);
			c = fgetc(temp_file);
		}
		fclose(temp_file);
		remove(temp_path);
	}
	return result;
}

int main(int argc, char** argv)
{
	int api_result = 0;

	if (argc != 3 && argc != 2) {
		fprintf(stderr, "How to use : xdwsmpl2 document [page]");
		return 0;
	}

	char in_path[_MAX_PATH];
	_fullpath(in_path, argv[1], _MAX_PATH);

	int page = 0;
	if (argc == 3) {
		page = atoi(argv[2]);
	}

	// Open the document handle.
	XDW_DOCUMENT_HANDLE h = NULL;
	XDW_OPEN_MODE_EX mode = {
		sizeof(XDW_OPEN_MODE_EX), XDW_OPEN_READONLY, XDW_AUTH_NODIALOGUE };
	api_result = XDW_OpenDocumentHandle(in_path, &h, (XDW_OPEN_MODE*)&mode);
	if (api_result < 0) {
		print_error(api_result);
		return 0;
	}

	// Use XDW_GetDocumentInformation to get the number of total pages.
	XDW_DOCUMENT_INFO info = { sizeof(XDW_DOCUMENT_INFO), 0, 0, 0 };
	XDW_GetDocumentInformation(h, &info);
	int last_page = info.nPages;

	if (argc == 3) {
		if (page < 1 || page > last_page) {
			fprintf(stderr, "Page not exist.");
		} else {
			api_result = print_text(h, page);
		}
	} else if (page == 0) {
		for (int i = 1; i <= last_page; i++) {
			api_result = print_text(h, i);
			if (api_result < 0) break;
		}
	}
	if (api_result < 0) print_error(api_result);

	// Close the document handle.
	XDW_CloseDocumentHandle(h, NULL);

	return (api_result >= 0);
}
