/* xdwapi sample 3

Function :
  Gets the strings of text annotations in the DocuWorks document at the standard output.
How to use:
  xdwsmpl3 document [page]

  document : DocuWorks document (in).
  page : Specified page (in). If omitted, all of pages are covered.
*/

#include <stdio.h>
#include <windows.h>

#include <xdw_api.h>
#include <xdwapian.h>

void print_error(int code)
{
	switch (code) {
	case XDW_E_NOT_INSTALLED:
		fprintf(stderr, "DocuWorks not installed.");
		break;
	case XDW_E_FILE_NOT_FOUND:
		fprintf(stderr, "File not found.");
		break;
	case XDW_E_FILE_EXISTS:
		fprintf(stderr, "File exists.");
		break;
	case XDW_E_ACCESSDENIED:
	case XDW_E_INVALID_NAME:
	case XDW_E_BAD_NETPATH:
		fprintf(stderr, "File not opened.");
		break;
	case XDW_E_BAD_FORMAT:
		fprintf(stderr, "File bad format.");
		break;
	case XDW_E_INVALID_ACCESS:
		fprintf(stderr, "Invalid access.");
		break;
	default:
		fprintf(stderr, "Error.");
		break;
	}
}

int print_text(XDW_ANNOTATION_HANDLE hAnn)
{
	// Use XDW_GetAnnotationAttribute to get the byte count required for the allocation.
	int size = 	XDW_GetAnnotationAttribute(hAnn, XDW_ATN_Text, NULL, 0, NULL);

	if (size > 0) {
		char *text = new char[size];
		// Use XDW_GetAnnotationAttribute to get the attribute value.
		XDW_GetAnnotationAttribute(hAnn, XDW_ATN_Text, text, size, NULL);
		puts(text);
		delete [] text;
	}
	return size;
}

int print_page(XDW_DOCUMENT_HANDLE h, int page)
{
	// Use XDW_GetPageInformation to get the number of annotations on the page.
	XDW_PAGE_INFO pInfo = { sizeof(XDW_PAGE_INFO), 0, 0, 0, 0, 0, 0, 0 };
	XDW_GetPageInformation(h, page, &pInfo);

	int result = 0;
	for (int i = 1; i <= pInfo.nAnnotations; i++) {
		// Use XDW_GetAnnotationInformation to get information of the annotation.
		XDW_ANNOTATION_INFO aInfo = {
			sizeof(XDW_ANNOTATION_INFO), NULL, 0, 0, 0, 0, 0, 0 };
		XDW_GetAnnotationInformation(h, page, NULL, i, &aInfo, NULL);

		// If annotations are attached on this annotaion,
		if (aInfo.nChildAnnotations > 0) {
			for (int j = 1; j <= aInfo.nChildAnnotations; j++) {
				// Use XDW_GetAnnotationInformation to get information of the child annotation.
				XDW_ANNOTATION_INFO aInfo2 = {
					sizeof(XDW_ANNOTATION_INFO), NULL, 0, 0, 0, 0, 0, 0 };
				XDW_GetAnnotationInformation(
					h, page, aInfo.handle, j, &aInfo2, NULL);
				if (aInfo2.nAnnotationType == XDW_AID_TEXT) {
					result = print_text(aInfo2.handle);
					if (result < 0) break;
				}
			}
		}
		
		else if (aInfo.nAnnotationType == XDW_AID_TEXT) {
			result = print_text(aInfo.handle);
			if (result < 0) break;
		}
	}
	return result;
}

int main(int argc, char** argv)
{
	int api_result = 0;

	if (argc != 3 && argc != 2) {
		fprintf(stderr, "How to use : xdwsmpl3 document [page]");
		return 0;
	}

	char in_path[_MAX_PATH];
	_fullpath(in_path, argv[1], _MAX_PATH);

	int page = 0;
	if (argc == 3) {
		page = atoi(argv[2]);
	}

	// Open the document handle.
	XDW_DOCUMENT_HANDLE h = NULL;
	XDW_OPEN_MODE_EX mode = {
		sizeof(XDW_OPEN_MODE_EX), XDW_OPEN_READONLY, XDW_AUTH_NODIALOGUE };
	api_result = XDW_OpenDocumentHandle(in_path, &h, (XDW_OPEN_MODE*)&mode);
	if (api_result < 0) {
		print_error(api_result);
		return 0;
	}

	// Use XDW_GetDocumentInformation to get the number of total pages.
	XDW_DOCUMENT_INFO info = { sizeof(XDW_DOCUMENT_INFO), 0 };
	XDW_GetDocumentInformation(h, &info);
	int last_page = info.nPages;

	if (argc == 3) {
		if (page < 1 || page > last_page) {
			fprintf(stderr, "Page not exist.");
		} else {
			api_result = print_page(h, page);
		}
	} else if (page == 0) {
		for (int i = 1; i <= last_page; i++) {
			api_result = print_page(h, i);
			if (api_result < 0) break;
		}
	}
	if (api_result < 0) print_error(api_result);

	// Close the document handle.
	XDW_CloseDocumentHandle(h, NULL);

	return (api_result >= 0);
}
